// AppLauncher の JavaScript

/**
 * 外部アプリケーションを起動する関数
 * 現在は、テスト用のアラートを表示するだけ.
 */
AppLauncher.launch = function( path, argsSource ) {
	try {
		// 設定値の取得
		//var path = AppLauncher.getPref("path");
		var argsList = "";
		var args = new Array();
		for( var i = 0; i < argsSource.length; i++ ) {
			var uri = this.getURL();
			// turi の設定
			// turi は右クリックのリンク先 URI. 右クリックしたのがリンク要素じゃなければそのページの URI
			var turi = null;
			var targetNode = document.popupNode;
			while ( targetNode != null && targetNode.nodeName.toUpperCase() != "A" ) {
				targetNode = targetNode.parentNode;
			}
			if ( targetNode != null && targetNode.nodeName.toUpperCase() == "A" ) {
				turi = targetNode.href;
			} else {
				turi = uri;
			}
			//window.alert( "turi: " + turi );
			args[i] = argsSource[i].replace( /&url;|&eurl;|&turl;/g, function(substr) {
				if ( substr == "&url;" ) { return uri; }
				else if ( substr == "&eurl;" ) { return encodeURIComponent(uri); }
				else if ( substr == "&turl;" ) { return turi; }
			} );
			// パターンの g は繰り返しオプション
			//args[i] = args[i].replace( /&eurl;/g,  );
			argsList = argsList + args[i] + " ";
		}
		// cf.https://developer.mozilla.org/ja/Code_snippets/Running_applications
		// 実行可能ファイルに対する nsILocalFile を作成する
		var file = Components.classes["@mozilla.org/file/local;1"]
								.createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(path);
		if( ! file.exists() ) {
			throw new Error("The File \"" + path + "\" doesn't exist!");
		}
		// nsIProcess を作成する 
		var process = Components.classes["@mozilla.org/process/util;1"]
								.createInstance(Components.interfaces.nsIProcess);
		process.init(file);
		// プロセスの起動
		// 最初のパラメータが true なら、スレッドはプロセスが終わるまでブロックされる。
		// 2 つ目と 3 つ目のパラメータは、プロセスのコマンドライン引数
		// var args = ["argument1", "argument2"];
		//var args = []; // とりあえず今は引数なし
		process.run(false, args, args.length);
	} catch(e) {
		window.alert("[EXCEPTION] An exception occured in \"AppLauncher\": \n" + e.message);
	}
};

/**
 * 現在表示中のページの URL を返す関数.
 */
AppLauncher.getURL = function() {
	/*
	// アクティブなウィンドウの取得
	// cf. https://developer.mozilla.org/Ja/Code_snippets/Tabbed_browser
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
					.getService(Components.interfaces.nsIWindowMediator);
	var recentWindow = wm.getMostRecentWindow("navigator:browser");
	return (recentWindow ? recentWindow.content.document.location.href : null);
	*/
	// ブラウザウィンドウ領域からなら以下で OK.
	return content.location.href;
}


/**
 * ユーザの設定画面に以前の設定を読み出す.
 */
AppLauncher.loadPrefs = function() {
	// 以前の設定を DOM として取得
	var appListStr = this.getPref("appList");
	var parser  = new DOMParser();
	var appList = parser.parseFromString(appListStr, "text/xml");
	var items   = appList.getElementsByTagNameNS(this.PREFS_NS, "app");
	// 設定画面の要素を取得
	var listbox = document.getElementById("info.vividcode.applauncher.prefwindow.listbox");
	//var items   = listbox.getElementsByTagNameNS(this.XUL_NS, "listitem");
	// 設定画面に要素を追加していく
	for( var i = 0; i < items.length; i++ ) {
		var item = document.createElementNS(this.XUL_NS, "listitem");
		var cell = document.createElementNS(this.XUL_NS, "listcell");
		cell.setAttribute( "label", items[i].getAttribute("name") );
		item.appendChild(cell);
		cell = document.createElementNS(this.XUL_NS, "listcell");
		cell.setAttribute( "label", items[i].getAttribute("path") );
		item.appendChild(cell);
		cell = document.createElementNS(this.XUL_NS, "listcell");
		cell.setAttribute( "label", items[i].getAttribute("args") );
		item.appendChild(cell);
		item.addEventListener("dblclick", this.elDeletingApp, false);
		listbox.appendChild(item);
	}
};

/**
 * メニュー項目に追加するイベントリスナを返す関数.
 */
AppLauncher.createElLaunch = function( pref ) {
	var path = pref.getPath();
	//var argsSource = prefElem.getAttribute("args").split("<>");
	var args = pref.getArgs();
	return function(evt) {
		//var args = new Array();
		//for( var i = 0; i < argsSource.length; i++ ) {
		//	args[i] = argsSource[i].replace("&url;", content.location.href);
		//}
		AppLauncher.launch(path, args);
	};
};

/**
 * 初期化関数
 * AppLauncher に関するコンテキストメニューの変更を行う.
 */
AppLauncher.init = function() {
	var prefs = info.vividcode.ext.applauncher.AppPref.loadAppPrefs();
	// コンテキストメニュー内の、AppLauncher に関する "menupopup" 要素 (id で指定) を取得
	var menupopup = document.getElementById("info.vividcode.applauncher.contextmenu.items");
	if( menupopup ) {
		if( prefs != null && prefs.length != 0 ) {
			for( var i = 0; i < prefs.length; i++ ) {
				// "menuitem" 要素の作成
				var item = document.createElementNS( this.XUL_NS, "menuitem" );
				// "label" 属性に "adding item!" という値を設定
				item.setAttribute( "label", prefs[i]._appPref.getName() );
				// イベントリスナの追加
				item.addEventListener("command", this.createElLaunch( prefs[i]._appPref ), false);
				// "menupopup" 要素の子ノードに追加
				menupopup.appendChild(item);
			}
		} else {
			// "menuitem" 要素の作成
			var item = document.createElementNS( this.XUL_NS, "menuitem" );
			// "label" 属性に "adding item!" という値を設定
			item.setAttribute( "label", "(外部アプリケーションが設定されていません...)" );
			// "menupopup" 要素の子ノードに追加
			menupopup.appendChild(item);
		}
		// "menuseparator" 要素を追加
		menupopup.appendChild( document.createElementNS( this.XUL_NS, "menuseparator" ) );
		// 設定画面を起動する要素を追加
		// "menuitem" 要素の作成
		var item = document.createElementNS( this.XUL_NS, "menuitem" );
		// "label" 属性に "adding item!" という値を設定
		item.setAttribute( "label", "設定" );
		// イベントリスナの追加
		item.addEventListener("command", function(e) {
			// 設定ウィンドウを表示
			window.open("chrome://applauncher/content/prefs.xul", "applauncherprefs", "chrome,dialog,resizable=yes"); 
		}, false);
		// "menupopup" 要素の子ノードに追加
		menupopup.appendChild(item);
	}
	/*
	// 以前の設定を DOM として取得
	var appListStr = this.getPref("appList");
	var items = null;
	try {
		var parser  = new DOMParser();
		var appList = parser.parseFromString(appListStr, "text/xml");
		if( appList ) {
			items   = appList.getElementsByTagNameNS(this.PREFS_NS, "app");
		}
	} catch(e) {
		// EXCEPTION
	}
	// コンテキストメニュー内の、AppLauncher に関する "menupopup" 要素 (id で指定) を取得
	var menupopup = document.getElementById("info.vividcode.applauncher.contextmenu.items");
	if( menupopup ) {
		if( items != null && items.length != 0 ) {
			for( var i = 0; i < items.length; i++ ) {
				// "menuitem" 要素の作成
				var item = document.createElementNS( this.XUL_NS, "menuitem" );
				// "label" 属性に "adding item!" という値を設定
				item.setAttribute( "label", items[i].getAttribute("name") );
				// イベントリスナの追加
				item.addEventListener("command", this.createElLaunch( items[i] ), false);
				// "menupopup" 要素の子ノードに追加
				menupopup.appendChild(item);
			}
		} else {
			// "menuitem" 要素の作成
			var item = document.createElementNS( this.XUL_NS, "menuitem" );
			// "label" 属性に "adding item!" という値を設定
			item.setAttribute( "label", "(外部アプリケーションが設定されていません...)" );
			// "menupopup" 要素の子ノードに追加
			menupopup.appendChild(item);
		}
		// "menuseparator" 要素を追加
		menupopup.appendChild( document.createElementNS( this.XUL_NS, "menuseparator" ) );
		// 設定画面を起動する要素を追加
		// "menuitem" 要素の作成
		var item = document.createElementNS( this.XUL_NS, "menuitem" );
		// "label" 属性に "adding item!" という値を設定
		item.setAttribute( "label", "設定" );
		// イベントリスナの追加
		item.addEventListener("command", function(e) {
			// 設定ウィンドウを表示
			window.open("chrome://applauncher/content/prefs.xul", "applauncherprefs", "chrome,dialog,resizable=yes"); 
		}, false);
		// "menupopup" 要素の子ノードに追加
		menupopup.appendChild(item);
	}
	*/
};

AppLauncher.elLoad = function(evt) {
	AppLauncher.init();
}

// 初期化関数をロード時に実行する
window.addEventListener("load", AppLauncher.elLoad, false);
